@@ -5,13 +5,14 @@ The inventree_api module handles low level requests and authentication
 with the InvenTree database server.
 """
 
-
 import requests
-from requests.auth import HTTPBasicAuth
 import os
 import json
 import logging
 
+from requests.auth import HTTPBasicAuth
+from urllib.parse import urljoin, urlparse
+
 
 logger = logging.getLogger('inventree')
 
@@ -21,7 +22,7 @@ class InvenTreeAPI(object):
     Basic class for performing Inventree API requests.
     """
 
-    MIN_SUPPORTED_API_VERSION = 6
+    MIN_SUPPORTED_API_VERSION = 51
 
     @staticmethod
     def getMinApiVersion():
@@ -31,7 +32,7 @@ class InvenTreeAPI(object):
 
         return InvenTreeAPI.MIN_SUPPORTED_API_VERSION
 
-    def __init__(self, base_url, **kwargs):
+    def __init__(self, host=None, **kwargs):
         """ Initialize class with initial parameters
 
         Args:
@@ -44,26 +45,68 @@ class InvenTreeAPI(object):
             token - Authentication token (if provided, username/password are ignored)
             use_token_auth - Use token authentication? (default = True)
             verbose - Print extra debug messages (default = False)
+
+        Login details can be specified using environment variables, rather than being provided as arguments:
+            INVENTREE_API_HOST - Host address e.g. "http://inventree.server.com:8000"
+            INVENTREE_API_USERNAME - Username
+            INVENTREE_API_PASSWORD - Password
+            INVENTREE_API_TOKEN - User access token
         """
 
-        # Strip out trailing "/api/" (if provided)
-        if base_url.endswith("/api/"):
-            base_url = base_url[:-5]
+        self.setHostName(host or os.environ.get('INVENTREE_API_HOST', None))
 
-        if not base_url.endswith('/'):
-            base_url += '/'
+        # Check for environment variables
+        self.username = kwargs.get('username', os.environ.get('INVENTREE_API_USERNAME', None))
+        self.password = kwargs.get('password', os.environ.get('INVENTREE_API_PASSWORD', None))
+        self.token = kwargs.get('token', os.environ.get('INVENTREE_API_TOKEN', None))
 
-        self.base_url = base_url
+        self.use_token_auth = kwargs.get('use_token_auth', True)
+        self.verbose = kwargs.get('verbose', False)
 
-        self.api_url = os.path.join(self.base_url, 'api/')
+        self.auth = None
+        self.connected = False
+
+        if kwargs.get('connect', True):
+            self.connect()
+
+    def setHostName(self, host):
+        """Validate that the provided base URL is valid"""
+        
+        if host is None:
+            raise AttributeError("InvenTreeAPI initialized without providing host address")
+
+        # Ensure that the provided URL is valid
+        url = urlparse(host)
+
+        if not url.scheme:
+            raise Exception(f"Host '{host}' supplied without valid scheme")
+        
+        if not url.netloc or not url.hostname:
+            raise Exception(f"Host '{host}' supplied without valid hostname")
+
+        # Check if the path is provided with '/api/' at the end
+        ps = [el for el in url.path.split('/') if len(el) > 0]
+
+        if len(ps) > 0 and ps[-1] == 'api':
+            ps = ps[:-1]
+
+        path = '/'.join(ps)
+
+        # Re-construct the URL as required
+        self.base_url = f"{url.scheme}://{url.netloc}/{path}"
+
+        if not self.base_url.endswith('/'):
+            self.base_url += '/'
+
+        # Re-construct the API URL as required
+        self.api_url = urljoin(self.base_url, 'api/')
+
+    def connect(self):
+        """Attempt a connection to the server"""
 
         logger.info(f"Connecting to server: {self.base_url}")
 
-        self.username = kwargs.get('username', None)
-        self.password = kwargs.get('password', None)
-        self.token = kwargs.get('token', None)
-        self.use_token_auth = kwargs.get('use_token_auth', True)
-        self.verbose = kwargs.get('verbose', False)
+        self.connected = False
 
         # Check if the server is there
         if not self.testServer():
@@ -75,11 +118,25 @@ class InvenTreeAPI(object):
         if self.use_token_auth:
             if not self.token:
                 self.requestToken()
+        
+        self.connected = True
 
-    def clean_url(self, url):
+    def constructApiUrl(self, endpoint_url):
+        """Construct an API endpoint URL based on the provided API URL.
 
-        url = os.path.join(self.api_url, url)
+        Arguments:
+            endpoint_url: The particular API endpoint (everything after "/api/")
 
+        Returns: A fully qualified URL for the subsequent request
+        """
+
+        # Strip leading / character if provided
+        if endpoint_url.startswith("/"):
+            endpoint_url = endpoint_url[1:]
+
+        url = urljoin(self.api_url, endpoint_url)
+
+        # Ensure the API URL ends with a trailing slash
         if not url.endswith('/'):
             url += '/'
 
@@ -99,10 +156,10 @@ class InvenTreeAPI(object):
         try:
             response = requests.get(self.api_url, timeout=2.5)
         except requests.exceptions.ConnectionError as e:
-            logger.fatal("Server connection error:", type(e))
+            logger.fatal(f"Server connection error: {str(type(e))}")
             return False
         except Exception as e:
-            logger.fatal("Unhandled server error:", type(e))
+            logger.fatal(f"Unhandled server error: {str(type(e))}")
             # Re-throw the exception
             raise e
 
@@ -130,6 +187,9 @@ class InvenTreeAPI(object):
         if api_version < InvenTreeAPI.getMinApiVersion():
             raise ValueError(f"Server API version ({api_version}) is older than minimum supported API version ({InvenTreeAPI.getMinApiVersion()})")
 
+        # Store the server API version
+        self.api_version = api_version
+
         return True
 
     def requestToken(self):
@@ -161,33 +221,28 @@ class InvenTreeAPI(object):
 
         return self.token
 
-    def request(self, url, **kwargs):
+    def request(self, api_url, **kwargs):
         """ Perform a URL request to the Inventree API """
 
-        # Remove leading slash
-        if url.startswith('/'):
-            url = url[1:]
+        if not self.connected:
+            # If we have not established a connection to the server yet, attempt now
+            self.connect()
 
-        api_url = os.path.join(self.api_url, url)
-
-        if not api_url.endswith('/'):
-            api_url += '/'
-
-        method = kwargs.get('method', 'get')
-
-        params = kwargs.get('params', {})
-
-        json = kwargs.get('json', {})
+        api_url = self.constructApiUrl(api_url)
 
+        data = kwargs.get('data', kwargs.get('json', {}))
         files = kwargs.get('files', {})
-
+        params = kwargs.get('params', {})
         headers = kwargs.get('headers', {})
 
-        search_term = kwargs.get('search', None)
+        search_term = kwargs.pop('search', None)
 
         if search_term is not None:
             params['search'] = search_term
 
+        # Use provided HTTP method
+        method = kwargs.get('method', 'get')
+
         methods = {
             'GET': requests.get,
             'POST': requests.post,
@@ -203,31 +258,37 @@ class InvenTreeAPI(object):
 
         method = method.upper()
 
+        payload = {
+            'params': params,
+            'timeout': kwargs.get('timeout', 10),
+        }
+
         if self.use_token_auth and self.token:
             headers['AUTHORIZATION'] = f'Token {self.token}'
             auth = None
         else:
             auth = self.auth
+        
+        payload['headers'] = headers
+        payload['auth'] = auth
 
+        # If we are providing files, we cannot upload as a 'json' request
+        if files:
+            payload['data'] = data
+            payload['files'] = files
+        else:
+            payload['json'] = data
+
+        # Debug request information
         logger.debug("Sending Request:")
         logger.debug(f" - URL: {method} {api_url}")
-        logger.debug(f" - auth: {auth}")
-        logger.debug(f" - params: {params}")
-        logger.debug(f" - headers: {headers}")
-        logger.debug(f" - json: {json}")
-        logger.debug(f" - files: {files}")
+
+        for item, value in payload.items():
+            logger.debug(f" - {item}: {value}")
 
         # Send request to server!
         try:
-            response = methods[method](
-                api_url,
-                auth=auth,
-                params=params,
-                headers=headers,
-                json=json,
-                files=files,
-                timeout=10,
-            )
+            response = methods[method](api_url, **payload)
         except Exception as e:
             # Re-thrown any caught errors, and add a message to the log
             logger.critical(f"Error at api.request - {method} @ {api_url}")
@@ -260,8 +321,8 @@ class InvenTreeAPI(object):
             if files:
                 detail['files'] = files
             
-            if json:
-                detail['data'] = json
+            if data:
+                detail['data'] = data
 
             raise requests.exceptions.HTTPError(detail)
 
@@ -298,7 +359,7 @@ class InvenTreeAPI(object):
 
         return response
 
-    def post(self, url, data, files=None, **kwargs):
+    def post(self, url, data, **kwargs):
         """ Perform a POST request. Used to create a new record in the database.
 
         Args:
@@ -307,57 +368,15 @@ class InvenTreeAPI(object):
             files - Dict of file attachments
         """
 
-        url = self.clean_url(url)
-
-        headers = kwargs.get('headers', {})
-
-        if self.use_token_auth and self.token:
-            headers['AUTHORIZATION'] = f'Token {self.token}'
-            auth = None
-        else:
-            auth = self.auth
-
-        response = requests.post(url, data=data, headers=headers, auth=auth, files=files, **kwargs)
-
-        if response is None:
-            return None
-
-        if response.status_code not in [200, 201]:
-            logger.error(f"POST request failed at '{url}' - {response.status_code}")
-            logger.error(f"{response.text}")
-            return None
-        
-        try:
-            data = json.loads(response.text)
-        except json.decoder.JSONDecodeError:
-            logger.error(f"Error decoding JSON response - '{url}'")
-            return None
-
-        return data
-
-    def patch(self, url, data, files=None, **kwargs):
-        """
-        Perform a PATCH request.
-
-        Args:
-            url - API endpoint URL
-            data - JSON data
-            files - optional FILES struct
-        """
-
-        headers = kwargs.get('headers', {})
-
         params = {
-            'format': 'json',
+            'format': kwargs.pop('format', 'json')
         }
 
         response = self.request(
             url,
             json=data,
-            method='patch',
-            headers=headers,
+            method='post',
             params=params,
-            files=files,
             **kwargs
         )
 
@@ -377,7 +396,45 @@ class InvenTreeAPI(object):
 
         return data
 
-    def put(self, url, data, files=None, **kwargs):
+    def patch(self, url, data, **kwargs):
+        """
+        Perform a PATCH request.
+
+        Args:
+            url - API endpoint URL
+            data - JSON data
+            files - optional FILES struct
+        """
+
+        params = {
+            'format': kwargs.pop('format', 'json')
+        }
+
+        response = self.request(
+            url,
+            json=data,
+            method='patch',
+            params=params,
+            **kwargs
+        )
+
+        if response is None:
+            logger.error(f"PATCH returned null response at '{url}'")
+            return None
+
+        if response.status_code not in [200, 201]:
+            logger.error(f"PATCH request failed at '{url}' - {response.status_code}")
+            return None
+
+        try:
+            data = json.loads(response.text)
+        except json.decoder.JSONDecodeError:
+            logger.error(f"Error decoding JSON response - '{url}'")
+            return None
+
+        return data
+
+    def put(self, url, data, **kwargs):
         """
         Perform a PUT request. Used to update existing records in the database.
 
@@ -386,19 +443,15 @@ class InvenTreeAPI(object):
             data - JSON data to PUT
         """
 
-        headers = kwargs.get('headers', {})
-
         params = {
-            'format': 'json',
+            'format': kwargs.pop('format', 'json')
         }
 
         response = self.request(
             url,
             json=data,
             method='put',
-            headers=headers,
             params=params,
-            files=files,
             **kwargs
         )
 
@@ -418,13 +471,9 @@ class InvenTreeAPI(object):
         return data
 
     def get(self, url, **kwargs):
-        """ Perform a GET request
-
-        Args:
-            url - API url
-
-        kwargs:
+        """ Perform a GET request.
 
+        For argument information, refer to the 'request' method
         """
 
         response = self.request(url, method='get', **kwargs)
@@ -451,13 +500,10 @@ class InvenTreeAPI(object):
         - If the "destination" is a directory, use the filename of the remote URL
         """
 
-        # Check that the provided URL is "absolute"
-        if not url.startswith(self.base_url):
+        if url.startswith('/'):
+            url = url[1:]
 
-            if url.startswith('/'):
-                url = url[1:]
-
-            url = os.path.join(self.base_url, url)
+        url = urljoin(self.base_url, url)
 
         if os.path.exists(destination) and os.path.isdir(destination):
 
