@@ -4,8 +4,10 @@ import os
 import logging
 import json
 
+from . import api as inventree_api
 
-INVENTREE_PYTHON_VERSION = "0.7.3"
+
+INVENTREE_PYTHON_VERSION = "0.8.1"
 
 
 logger = logging.getLogger('inventree')
@@ -28,18 +30,23 @@ class InventreeObject(object):
         """ Instantiate this InvenTree object.
 
         Args:
-            pk - The ID (primary key) associated with this object on the server
             api - The request manager object
+            pk - The ID (primary key) associated with this object on the server
             data - JSON representation of the object
         """
 
         # If the pk is not explicitly provided,
         # extract it from the provided dataset
-        if pk is None:
+        if pk is None and data:
             pk = data.get('pk', None)
-        elif type(pk) is not int:
+        
+        # Convert to integer
+        try:
+            pk = int(pk)
+        except Exception:
             raise TypeError(f"Supplied <pk> value ({pk}) for {self.__class__} is invalid.")
-        elif pk <= 0:
+        
+        if pk <= 0:
             raise ValueError(f"Supplier <pk> value ({pk}) for {self.__class__} must be positive.")
 
         self._url = f"{self.URL}/{pk}/"
@@ -55,16 +62,16 @@ class InventreeObject(object):
             self.reload()
 
     @classmethod
-    def fields(cls, api):
-        """
-        Returns a list of available fields for this model.
+    def options(cls, api):
+        """Perform an OPTIONS request for this model, to determine model information.
 
-        Introspects the available fields using an OPTIONS request.
+        InvenTree provides custom metadata for each API endpoint, accessed via a HTTP OPTIONS request.
+        This endpoint provides information on the various fields available for that endpoint.
         """
 
         response = api.request(
             cls.URL,
-            method='options',
+            method='OPTIONS',
         )
 
         if not response.status_code == 200:
@@ -74,14 +81,38 @@ class InventreeObject(object):
         try:
             data = json.loads(response.text)
         except json.decoder.JSONDecodeError:
-            logger.error(f"Error decoding JSON response for '{cls.URL}'")
+            logger.error(f"Error decoding OPTIONS response for '{cls.URL}'")
             return {}
 
-        actions = data.get('actions', {})
+        return data
+
+    @classmethod
+    def fields(cls, api):
+        """
+        Returns a list of available fields for this model.
+
+        Introspects the available fields using an OPTIONS request.
+        """
+
+        opts = cls.options(api)
+
+        actions = opts.get('actions', {})
         post = actions.get('POST', {})
 
         return post
 
+    @classmethod
+    def fieldInfo(cls, field_name, api):
+        """Return metadata for a specific field on a model"""
+
+        fields = cls.fields(api)
+
+        if field_name in fields:
+            return fields[field_name]
+        else:
+            logger.warning(f"Field '{field_name}' not found in OPTIONS request for {cls.URL}")
+            return {}
+
     @classmethod
     def fieldNames(cls, api):
         """
@@ -93,7 +124,14 @@ class InventreeObject(object):
     @property
     def pk(self):
         """ Convenience method for accessing primary-key field """
-        return self._data.get('pk', None)
+        val = self._data.get('pk', None)
+
+        try:
+            val = int(val)
+        except ValueError:
+            pass
+    
+        return val
 
     @classmethod
     def create(cls, api, data, **kwargs):
@@ -215,13 +253,14 @@ class InventreeObject(object):
                 logger.error(f"Error during reload at {self._url} - returned data is invalid")
 
         else:
-            raise ValueError(f"model.reload failed at '{self._url}': No API instance provided")
+            raise AttributeError(f"model.reload failed at '{self._url}': No API instance provided")
 
     def __getattr__(self, name):
+
         if name in self._data.keys():
             return self._data[name]
         else:
-            return super().__getattr__(name)
+            return super().__getattribute__(name)
 
     def __getitem__(self, name):
         if name in self._data.keys():
@@ -236,11 +275,58 @@ class InventreeObject(object):
             raise KeyError(f"Key '{name}' does not exist in dataset")
 
 
-class Attachment(InventreeObject):
+class BulkDeleteMixin:
+    """Mixin class for models which support 'bulk deletion'
+
+    - Perform a DELETE operation against the LIST endpoint for the model
+    - Provide a list of items to be deleted, or filters to apply
+
+    Requires API version 58
+    """
+
+    @classmethod
+    def bulkDelete(cls, api: inventree_api.InvenTreeAPI, items=None, filters=None):
+        """Perform bulk delete operation
+
+        Arguments:
+            api: InventreeAPI instance
+            items: Optional list of items (pk values) to be deleted
+            filters: Optional query filters to delete
+
+        Returns:
+            API response object
+        
+        Throws:
+            NotImplementError: The server API version is too old (requires v58)
+            ValueError: Neither items or filters are supplied
+
+        """
+
+        if api.api_version < 58:
+            raise NotImplementedError("bulkDelete requires API version 58 or newer")
+
+        if not items and not filters:
+            raise ValueError("Must supply either 'items' or 'filters' argument")
+        
+        data = {}
+
+        if items:
+            data['items'] = items
+        
+        if filters:
+            data['filters'] = filters
+
+        return api.delete(
+            cls.URL,
+            json=data,
+        )
+
+
+class Attachment(BulkDeleteMixin, InventreeObject):
     """
     Class representing a file attachment object
     
-    Multiple sub-classes exist, representing various types of attachment models in the database
+    Multiple sub-classes exist, representing various types of attachment models in the database.
     """
 
     # List of required kwargs required for the particular subclass
@@ -314,10 +400,75 @@ class Currency(InventreeObject):
     URL = 'common/currency'
 
 
-class ImageMixin:
+class MetadataMixin:
+    """Mixin class for models which support a 'metadata' attribute.
+
+    - The 'metadata' is not used for any InvenTree business logic
+    - Instead it can be used by plugins for storing arbitrary information
+    - Internally it is stored as a JSON database field
+    - Metadata is accessed via the API by appending '/metadata/' to the API URL
+
+    Note: Requires server API version 49 or newer
+
     """
-    Mixin class for supporting image upload against a model,
-    which has a specific 'image' field associated
+
+    @property
+    def metdata_url(self):
+        return os.path.join(self._url, "metadata/")
+
+    def getMetadata(self):
+        """Read model instance metadata"""
+        if self._api:
+
+            if self._api.api_version < 49:
+                logger.error("API version 49 or newer required to access instance metadata")
+                return {}
+
+            response = self._api.get(self.metdata_url)
+
+            return response['metadata']
+        else:
+            raise AttributeError(f"model.getMetadata failed at '{self._url}': No API instance provided")
+
+    def setMetadata(self, data, overwrite=False):
+        """Write metadata to this particular model.
+        
+        Arguments:
+            data: The data to be written. Must be a dict object
+            overwrite: If true, provided data replaces existing data. If false (default) data is merged with any existing data.
+        """
+
+        if type(data) is not dict:
+            raise TypeError("Data provided to 'setMetadata' method must be a dict object")
+
+        if self._api:
+
+            if self._api.api_version < 49:
+                logger.error("API version 49 or newer required to access instance metadata")
+                return None
+
+            if overwrite:
+                return self._api.put(
+                    self.metdata_url,
+                    data={
+                        "metadata": data,
+                    }
+                )
+            else:
+                return self._api.patch(
+                    self.metdata_url,
+                    data={
+                        "metadata": data
+                    }
+                )
+        else:
+            raise AttributeError(f"model.setMetadata failed at '{self._url}': No API instance provided")
+
+
+class ImageMixin:
+    """Mixin class for supporting image upload against a model.
+
+    - The model must have a specific 'image' field associated
     """
 
     def uploadImage(self, image):
