@@ -12,7 +12,21 @@ import inventree.build
 logger = logging.getLogger('inventree')
 
 
-class PartCategory(inventree.base.InventreeObject):
+class PartCategoryParameterTemplate(inventree.base.InventreeObject):
+    """A model which link a ParameterTemplate to a PartCategory"""
+
+    URL = 'part/category/parameters'
+
+    def getCategory(self):
+        """Return the referenced PartCategory instance"""
+        return PartCategory(self._api, self.category)
+    
+    def getTemplate(self):
+        """Return the referenced ParameterTemplate instance"""
+        return ParameterTemplate(self._api, self.parameter_template)
+
+
+class PartCategory(inventree.base.MetadataMixin, inventree.base.InventreeObject):
     """ Class representing the PartCategory database model """
 
     URL = 'part/category'
@@ -29,19 +43,21 @@ class PartCategory(inventree.base.InventreeObject):
     def getChildCategories(self, **kwargs):
         return PartCategory.list(self._api, parent=self.pk, **kwargs)
 
-    def get_category_parameter_templates(self, fetch_parent=True):
-        """
-            fetch_parent: enable to fetch templates for parent categories
+    def getCategoryParameterTemplates(self, fetch_parent: bool = True) -> list:
+        """Fetch a list of default parameter templates associated with this category
+
+        Arguments:
+            fetch_parent: If True (default) include templates for parents also
         """
 
-        parameters_url = f'part/category/{self.pk}/parameters'
-
-        return self.list(self._api,
-                         url=parameters_url,
-                         fetch_parent=fetch_parent)
+        return PartCategoryParameterTemplate.list(
+            self._api,
+            category=self.pk,
+            fetch_parent=fetch_parent
+        )
 
 
-class Part(inventree.base.ImageMixin, inventree.base.InventreeObject):
+class Part(inventree.base.MetadataMixin, inventree.base.ImageMixin, inventree.base.InventreeObject):
     """ Class representing the Part database model """
 
     URL = 'part'
