@@ -4,6 +4,8 @@ import os
 import requests
 import sys
 
+from requests.exceptions import HTTPError
+
 try:
     import Image
 except ImportError:
@@ -13,41 +15,12 @@ sys.path.append(os.path.abspath(os.path.dirname(__file__)))
 
 from test_api import InvenTreeTestCase  # noqa: E402
 
-from inventree.part import Part, PartAttachment, PartCategory, Parameter, ParameterTemplate  # noqa: E402
+from inventree.part import Part, PartAttachment, PartCategory, PartCategoryParameterTemplate, Parameter, ParameterTemplate  # noqa: E402
 from inventree.part import InternalPrice  # noqa: E402
 
 
-class PartTest(InvenTreeTestCase):
-    """
-    Test for PartCategory and Part objects.
-    """
-
-    def test_access_erors(self):
-        """
-        Test that errors are flagged when we try to access an invalid part
-        """
-
-        with self.assertRaises(TypeError):
-            Part(self.api, 'hello')
-        
-        with self.assertRaises(ValueError):
-            Part(self.api, -1)
-
-        # Try to access a Part which does not exist
-        with self.assertRaises(requests.exceptions.HTTPError):
-            Part(self.api, 9999999999999)
-
-    def test_fields(self):
-        """
-        Test field names via OPTIONS request
-        """
-
-        field_names = Part.fieldNames(self.api)
-
-        self.assertIn('active', field_names)
-        self.assertIn('revision', field_names)
-        self.assertIn('full_name', field_names)
-        self.assertIn('IPN', field_names)
+class PartCategoryTest(InvenTreeTestCase):
+    """Tests for PartCategory models"""
 
     def test_part_cats(self):
         """
@@ -145,6 +118,105 @@ class PartTest(InvenTreeTestCase):
 
         self.assertEqual(len(parts), n_parts + 10)
 
+    def test_part_category_parameter_templates(self):
+        """Unit tests for the PartCategoryParameterTemplate model"""
+
+        electronics = PartCategory(self.api, pk=3)
+
+        # Ensure there are some parameter templates associated with this category
+        templates = electronics.getCategoryParameterTemplates(fetch_parent=False)
+
+        if len(templates) == 0:
+            for name in ['wodth', 'lungth', 'herght']:
+                template = ParameterTemplate.create(self.api, data={
+                    'name': name,
+                    'units': 'uu',
+                })
+
+                pcpt = PartCategoryParameterTemplate.create(
+                    self.api,
+                    data={
+                        'category': electronics.pk,
+                        'parameter_template': template.pk,
+                        'default_value': name,
+                    }
+                )
+
+                # Check that model lookup functions work
+                self.assertEqual(pcpt.getCategory().pk, electronics.pk)
+                self.assertEqual(pcpt.getTemplate().pk, template.pk)
+
+            # Reload
+            templates = electronics.getCategoryParameterTemplates(fetch_parent=False)
+
+        self.assertTrue(len(templates) >= 3)
+
+        # Check child categories
+        childs = electronics.getChildCategories()
+
+        self.assertTrue(len(childs) > 0)
+
+        for child in childs:
+            child_templates = child.getCategoryParameterTemplates(fetch_parent=True)
+            self.assertTrue(len(child_templates) >= 3)
+
+
+class PartTest(InvenTreeTestCase):
+    """Tests for Part models"""
+
+    def test_access_erors(self):
+        """
+        Test that errors are flagged when we try to access an invalid part
+        """
+
+        with self.assertRaises(TypeError):
+            Part(self.api, 'hello')
+        
+        with self.assertRaises(ValueError):
+            Part(self.api, -1)
+
+        # Try to access a Part which does not exist
+        with self.assertRaises(requests.exceptions.HTTPError):
+            Part(self.api, 9999999999999)
+
+    def test_fields(self):
+        """
+        Test field names via OPTIONS request
+        """
+
+        field_names = Part.fieldNames(self.api)
+
+        self.assertIn('active', field_names)
+        self.assertIn('revision', field_names)
+        self.assertIn('full_name', field_names)
+        self.assertIn('IPN', field_names)
+
+    def test_options(self):
+        """Extends tests for OPTIONS model metadata"""
+
+        # Check for field which does not exist
+        with self.assertLogs():
+            Part.fieldInfo('abcde', self.api)
+
+        active = Part.fieldInfo('active', self.api)
+
+        self.assertEqual(active['type'], 'boolean')
+        self.assertEqual(active['required'], True)
+        self.assertEqual(active['label'], 'Active')
+        self.assertEqual(active['default'], True)
+
+        for field_name in [
+            'name',
+            'description',
+            'component',
+            'assembly',
+        ]:
+            field = Part.fieldInfo(field_name, self.api)
+
+            # Check required field attributes
+            for attr in ['type', 'required', 'read_only', 'label', 'help_text']:
+                self.assertIn(attr, field)
+
     def test_pagination(self):
         """ Test that we can paginate the queryset by specifying a 'limit' parameter"""
 
@@ -441,10 +513,8 @@ class PartTest(InvenTreeTestCase):
         existingTemplates = len(ParameterTemplate.list(self.api))
         
         # Create new parameter template - this will fail, no name given
-        parametertemplate = ParameterTemplate.create(self.api, data={'units': "kg A"})
-        
-        # result should be None
-        self.assertIsNone(parametertemplate)
+        with self.assertRaises(HTTPError):
+            parametertemplate = ParameterTemplate.create(self.api, data={'units': "kg A"})
         
         # Now create a proper parameter template
         parametertemplate = ParameterTemplate.create(self.api, data={'name': f'Test parameter no {existingTemplates}', 'units': "kg A"})
@@ -462,16 +532,12 @@ class PartTest(InvenTreeTestCase):
         existingParameters = len(p.getParameters())
         
         # Define parameter value for this part - without all required values
-        param = Parameter.create(self.api, data={'part': p.pk, 'template': parametertemplate.pk})
-        
-        # result should be None
-        self.assertIsNone(param)
-        
+        with self.assertRaises(HTTPError):
+            Parameter.create(self.api, data={'part': p.pk, 'template': parametertemplate.pk})
+            
         # Define parameter value for this part - without all required values
-        param = Parameter.create(self.api, data={'part': p.pk, 'data': 10})
-        
-        # result should be None
-        self.assertIsNone(param)
+        with self.assertRaises(HTTPError):
+            Parameter.create(self.api, data={'part': p.pk, 'data': 10})
         
         # Define w. required values - integer
         param = Parameter.create(self.api, data={'part': p.pk, 'template': parametertemplate.pk, 'data': 10})
@@ -481,10 +547,8 @@ class PartTest(InvenTreeTestCase):
         
         # Same parameter for same part - should fail
         # Define w. required values - string
-        param2 = Parameter.create(self.api, data={'part': p.pk, 'template': parametertemplate.pk, 'data': 'String value'})
-        
-        # result should be None
-        self.assertIsNone(param2)
+        with self.assertRaises(HTTPError):
+            Parameter.create(self.api, data={'part': p.pk, 'template': parametertemplate.pk, 'data': 'String value'})
         
         # Number of parameters should be one higher than before
         self.assertEqual(len(p.getParameters()), existingParameters + 1)
@@ -500,3 +564,42 @@ class PartTest(InvenTreeTestCase):
         
         # Check count
         self.assertEqual(len(ParameterTemplate.list(self.api)), existingTemplates)
+
+    def test_metadata(self):
+        """Test Part instance metadata"""
+
+        # Grab the first available part
+        part = Part.list(self.api, limit=1)[0]
+
+        part.setMetadata(
+            {
+                "foo": "bar",
+            },
+            overwrite=True,
+        )
+
+        metadata = part.getMetadata()
+
+        # Check that the metadata has been overwritten
+        self.assertEqual(len(metadata.keys()), 1)
+
+        self.assertEqual(metadata['foo'], 'bar')
+
+        # Now 'patch' in some metadata
+        part.setMetadata(
+            {
+                'hello': 'world',
+            },
+        )
+
+        part.setMetadata(
+            {
+                'foo': 'rab',
+            }
+        )
+
+        metadata = part.getMetadata()
+
+        self.assertEqual(len(metadata.keys()), 2)
+        self.assertEqual(metadata['foo'], 'rab')
+        self.assertEqual(metadata['hello'], 'world')
